function [Delta,G_iterated] = ComputeDelta(X,Density,G,PeakOpts)

% For each point, compute the distance to the nearest neighbor of higher
% density

Delta=zeros(size(Density));

t=PeakOpts.DiffusionTime;

if strcmp(PeakOpts.ModeDetection,'Euclidean')
    NN_List=G.DistInfo.idxs;
    NN_DistList=G.DistInfo.dists;
    G_iterated=[];
elseif strcmp(PeakOpts.ModeDetection,'Diffusion')
    WeightedEigVecs=(G.EigenVals.^t)'.*G.EigenVecs;
    G_iterated=DiffusionDistance(WeightedEigVecs,PeakOpts.DiffusionOpts);
    NN_List=G_iterated.DistInfo.idxs;
    NN_DistList=G_iterated.DistInfo.dists;
end

MaxDensity=max(Density);
for i=1:length(Density)
    NeighborsHigherDensity{i}=find(Density(i)<Density(NN_List{i}));
    if ~isempty(NeighborsHigherDensity{i})
        Delta(i)=NN_DistList{i}(NeighborsHigherDensity{i}(1));
    elseif Density(i)==MaxDensity
        if strcmp(PeakOpts.ModeDetection,'Euclidean')
            Delta(i)=max(pdist2(X(i,:),X));
        elseif strcmp(PeakOpts.ModeDetection,'Diffusion')
            Delta(i)=max(pdist2(WeightedEigVecs(i,:),WeightedEigVecs));
        end
    elseif isempty(NeighborsHigherDensity{i}) && Density(i)<MaxDensity
        if strcmp(PeakOpts.ModeDetection,'Euclidean')
            PWdist=pdist2(X(i,:),X);
        elseif strcmp(PeakOpts.ModeDetection,'Diffusion')
            PWdist=pdist2(WeightedEigVecs(i,:),WeightedEigVecs);
        end
        Delta(i)=min(PWdist(Density>Density(i)));  
    end
end



end