function [CoresLearned, CoreLabels, Centers, G,ExtendedCenters, DistStruct, G_iterated]=DensityPeaksEstimation(X, K, CoreSize, PeakOpts)

% DeltaDensity is the product of a kernel density estimator and a measure
% of distance from the point to a point of higher kernel density.  Points
% with large values should have both high densities and be far from points
% with higher densities.

[DeltaDensity, Density, Delta, G, G_iterated]=FindDensityPeaks(X,PeakOpts);

DistStruct.DeltaDensity=DeltaDensity;
DistStruct.Density=Density;
DistStruct.Delta=Delta;
DistStruct.G=G;


% Use the largest delta density values to learn cores.

[~,idx]=sort(DeltaDensity,'descend');


Centers(1)=idx(1);
Needed=K-1;
start=2;
% This is a technical script to make sure the cores that are learned are
% not duplicates.

while Needed>0
    try
        if abs(DeltaDensity(idx(start))-DeltaDensity(Centers(K-Needed)))>1e-7
            Centers(end+1)=idx(start);
            Needed=Needed-1;
        end
        start=start+1;
    catch
        keyboard
    end
end

if PeakOpts.UserPlot==1
    ExtendedCenters(1)=idx(1);
    
    Needed=2*K-1;
    start=2;
    % This is a technical script to make sure the cores that are learned are
    % not duplicates.
    
    while Needed>0
        try
            if abs(DeltaDensity(idx(start))-DeltaDensity(ExtendedCenters(2*K-Needed)))>1e-7
                ExtendedCenters(end+1)=idx(start);
                Needed=Needed-1;
            end
            start=start+1;
        catch
            
            keyboard
        end
    end
    
else
    ExtendedCenters=[];
end

%% Determine cores and give labels to the cores

CoresLearned=zeros(CoreSize,K);
CoreLabels=zeros(CoreSize*K,K);

if strcmp(PeakOpts.ModeDetection,'Euclidean')
    PWdist=pdist2(X(Centers,:),X);
elseif strcmp(PeakOpts.ModeDetection,'Diffusion')
    G=DistStruct.G;
    t=PeakOpts.DiffusionTime;
    WeightedEigVecs=(G.EigenVals.^t)'.*G.EigenVecs;
    PWdist=pdist2(WeightedEigVecs(Centers,:),WeightedEigVecs);
end

for k=1:K
    [~,idx]=sort(PWdist(k,:),'ascend');
    CoresLearned(:,k)=idx(1:CoreSize);
    CoreLabels(CoreSize*(k-1)+1:CoreSize*k,k)=1;
end

end
