% Compute the density peaks of the dataset X.  Compute the true finite-
% sample density and the distances Delta from each point to the
% point of higher density.

function [DeltaDensity, Density, Delta, G, G_iterated] = FindDensityPeaks(X,PeakOpts)

% Compute pairwise distances to be used for Delta computation

G=DiffusionDistance(X,PeakOpts.DiffusionOpts);

% Compute density and initialize Delta

Density=LocalDensity(X,G,PeakOpts);

% Compute Delta by measuring the minimal distance to a point of higher density

[Delta, G_iterated] = ComputeDelta(X,Density,G,PeakOpts);

% Normalize 

Delta=Delta/max(Delta);
Density=Density/sum(Density);

% Combine these measures multiplicatively, as suggested in FSFDPC paper (Science, 2014)

DeltaDensity=Delta.*(Density);

end