% Compute a clustering as described in the original fast-search-and-find
% of density peaks paper, then regularize by spatial consensus.

function [Labels,Labels_Pass1] = SpatialSpectralDiffusionLearning(X,K,DistStruct,Centers,I,J,M,N,WindowSize,PeakOpts, G_iterated)
if WindowSize<1
    Labels=FSFclustering(X,K,DistStruct,Centers);
end

Labels=zeros(1,size(X,1));

Density=DistStruct.Density;

if isempty(Centers)
    
    PWdist=squareform(pdist(X)); %Pairwise distances
    
    [DeltaDensity, Density,~] = FindDensityPeaks(X,K,PWdist,opts);
    
    [~,idx]=sort(DeltaDensity,'descend');
    
    Centers(1)=idx(1);
    
    Needed=K-1;
    start=2;
    
    % This is a technical script to make sure the cores that are learned are
    % not duplicates.
    
    while Needed>0
        if abs(DeltaDensity(idx(start))-DeltaDensity(Centers(K-Needed)))>1e-6
            Centers(end+1)=idx(start);
            Needed=Needed-1;
        end
        start=start+1;
    end
end

for k=1:K
    Labels(Centers(:,k))=k;
end

[~,idx]=sort(Density,'descend');
G=DistStruct.G;
t=PeakOpts.DiffusionTime;
WeightedEigVecs=(G.EigenVals.^t)'.*G.EigenVecs;

%% First pass

LabelsMatrix=full(sparse(I,J,Labels,M,N));

for j=1:length(Labels)
    if Labels(idx(j))==0
        NN=FindNearestLabeledNeighbor(X,j,idx,G,G_iterated,PeakOpts);
        NN=find(idx==NN);
        Labels(idx(j))=Labels(idx(NN));
        % Special case when many value have same density
        if Labels(idx(j))==0
            temp=find(Labels>0);
            if strcmp(PeakOpts.ModeDetection,'Euclidean')
                PWdist=pdist2(X(idx(j),:),X(temp,:));
            elseif strcmp(PeakOpts.ModeDetection,'Diffusion')
                PWdist=pdist2(WeightedEigVecs(idx(j),:),WeightedEigVecs(temp,:));
            end
            [~,NN]=min(PWdist);
            Labels(idx(j))=Labels(temp(NN));
        end
        % Check spatial consensus
        LabelsMatrix(I(idx(j)),J(idx(j)))=Labels(idx(j));
        SpatialConsensusLabel=SpatialConsensus(LabelsMatrix,idx(j),M,N,I,J,WindowSize);
        if ~(SpatialConsensusLabel==Labels(idx(j))) && (SpatialConsensusLabel>0)
            Labels(idx(j))=0;
            LabelsMatrix(I(idx(j)),J(idx(j)))=0;
        end
    end
end

Labels_Pass1=Labels;

%% Second pass

for j=1:length(Labels)
    if Labels(idx(j))==0
        NN=FindNearestLabeledNeighbor(X,j,idx,G,G_iterated,PeakOpts);
        NN=find(idx==NN);
        Labels(idx(j))=Labels(idx(NN));
        % Special case when many value have same density
        if Labels(idx(j))==0
            temp=find(Labels>0);
            if strcmp(PeakOpts.ModeDetection,'Euclidean')
                PWdist=pdist2(X(idx(j),:),X(temp,:));
            elseif strcmp(PeakOpts.ModeDetection,'Diffusion')
                PWdist=pdist2(WeightedEigVecs(idx(j),:),WeightedEigVecs(temp,:));
            end
            [~,NN]=min(PWdist);
            Labels(idx(j))=Labels(temp(NN));
        end
        % Enforce spatial consensus if obvious
        LabelsMatrix=sparse(I(idx(1:j)),J(idx(1:j)),Labels(idx(1:j)),M,N);
        [SpatialConsensusLabel,ConsensusFrac]=SpatialConsensus(LabelsMatrix,idx(j),M,N,I,J,WindowSize);
        if ConsensusFrac>.5
            Labels(idx(j))=SpatialConsensusLabel;
        end
    end
end

end

