% Compute labels for each method.

%% Some methods (active learning) require GT.  If it does not exist, make a zero matrix for it.

if isempty(GT)
    [I,J]=find(sum(DataParameters.RawHSI,3)~=0);
else
   [I,J]=find(GT>-1);
end

%% Kmeans on raw data

tic;
[LabelsRaw,C]=kmeans(X,K_Learned,'Replicates', NumReplicates, 'MaxIter', MaxIterations);
LabelsRaw=LabelsRaw';
TimeRaw=toc;

%% Kmeans PCA

tic;
LabelsPCA=kmeans(X*V(:,1:K_Learned),K_Learned,'Replicates', NumReplicates, 'MaxIter', MaxIterations)';
TimeICA=toc+TimeICA;

%% Kmeans on IC

LabelsICA=kmeans(ICA',K_Learned,'Replicates', NumReplicates, 'MaxIter', MaxIterations)';

%% Kmeans random projection

tic;
LabelsRandProj=kmeans(X*RP,K_Learned,'Replicates', NumReplicates, 'MaxIter', MaxIterations)';
TimeRP=toc+TimeRP;

%% Gaussian mixture model

tic;
LabelsGMM=ClusterGmm(X,K_GT)';
TimeGMM=toc;

%% Spectral clustering

tic;
LabelsSpectral=kmeans(EigVecsLE,K_Learned,'Replicates', NumReplicates, 'MaxIter', MaxIterations)';
TimeSC=toc+TimeSC;

%% SMCE

tic;
LabelsSMCE=kmeans(EigVecsSMCE,K_Learned,'Replicates', NumReplicates, 'MaxIter', MaxIterations)';
TimeSMCE=toc+TimeSMCE;

%% Hierarchical NMF

tic;
LabelsNMF=hierclust2nmf((X-min(min(X)))',K_Learned);
TimeNMF=toc;

%% FSFDPC

tic;
PeakOpts.ModeDetection='Euclidean';
LabelsFSFDPC=FSFclustering(X,K_GT,DistStructEuclidean,CentersEuclidean,PeakOpts,[]);
TimeFSFDPC=toc+TimeFSFDPC_1;


%% Diffusion Learning

tic;
PeakOpts.ModeDetection='Diffusion';
Labels_DL=FSFclustering(X,K_GT,DistStructDiffusion,CentersDiffusion,PeakOpts,G_iterated);
TimeDL=toc+TimeDL_1;

%% Spatial-Spectral Diffusion Learning

tic;
[Labels_DLSS,Labels_DLSS_Pass1]=SpatialSpectralDiffusionLearning(X,K_GT,DistStructDiffusion,CentersDiffusion,I,J,M,N,SpatialWindowSize,PeakOpts,G_iterated);
TimeDLSS=toc+TimeDL_1;

%% Active Spatial-Spectral Diffusion Learning

tic;
[Labels_DLSSActive,Labels_DLSS_Pass1Active,IdxModeDist]=SpatialSpectralDiffusionLearningActive(X,K_GT,DistStructDiffusion,CentersDiffusion,I,J,M,N,SpatialWindowSize,ActivePct,GT,ActiveMethod,PeakOpts,G_iterated);
TimeDLSS_Active=toc+TimeDL_1;

%% PLSR Euclidean

%Maximal label for PLSR regression with learned cores via Euclidean distance as training labels
[~,LabelsPLSR_EuclideanCores]=max(ResponsesPLSRCoresEuclidean,[],2);
LabelsPLSR_EuclideanCores=LabelsPLSR_EuclideanCores';

%% PLSR Diffusion

%Maximal label for PLSR regression with learned cores via diffusion distance as training labels
[~,LabelsPLSR_DiffusionCores]=max(ResponsesPLSRCoresDiffusion,[],2);
LabelsPLSR_DiffusionCores=LabelsPLSR_DiffusionCores';
