% Test DL and DLSS against a variety of comparison methods.

profile off;
profile on;

clear all;

%% Ask user to choose dataset

prompt = 'Which dataset? \n 1.) Indian Pines \n 2.) Pavia \n 3.) Salinas A \n';
DataSetSelected = input(prompt);

if DataSetSelected==1
    DataInput.datasource='HSI';
    DataInput.HSIname='IP'; 
    
elseif DataSetSelected==2
    DataInput.datasource='HSI';
    DataInput.HSIname='Pavia';
    
elseif DataSetSelected==3
    DataInput.datasource='HSI';
    DataInput.HSIname='SalinasA';
    
end
    
[X,Y,Ytuple,K_GT,d,DataParameters]=GenerateData(DataInput);
GT=DataParameters.GT;
M=DataParameters.M;
N=DataParameters.N;

%% Set Parameters

%Data name
PeakOpts.Data=DataInput.HSIname;

%Should we plot everything or not?  Should we save things?
UserPlot=0;
SavePlots=0;
PeakOpts.UserPlot=UserPlot;

%Detect number of eigenvectors to use automatically
PeakOpts.DiffusionOpts.K='automatic'; 

%How many nearest neighbors to use for diffusion distance
PeakOpts.DiffusionOpts.kNN=100;

%Force probability of self-loop to exceed .5.
PeakOpts.DiffusionOpts.LazyWalk=0;

%Set epsilon for weight matrix; use same one for spectral clustering
PeakOpts.DiffusionOpts.epsilon=1;

% Set time to evaluate diffusion distance
PeakOpts.DiffusionTime=30;

%Kmeans parameters
NumReplicates=10;
MaxIterations=100;

%Set how many cores we want to learn.  If K_Learned is different from
%K_GT, it will not be possible to compare easily and immediately against
%the ground truth, though other patterns may be studied in this way.
K_Learned=K_GT;

%How large should the cores be?  
%CoreSize=ceil(.02*(size(X,1)));
CoreSize=100;

%Spatial window size for labeling regime
SpatialWindowSize=3;

%Active learning percentage; this is the percent of pixels used for the
%active learning stage of the analysis.
ActivePct=.002;

% Method of active learning
ActiveMethod='Principled';

%%  Randomly select true labels for each class and perform density peaks estimation to learn labels.  With diffusion distance.

if ~isempty(GT)
    [GTLabels, GTLabelsTuple] = GenerateLabelsGT(Ytuple,CoreSize,K_GT);
end

%% Learn modes with Euclidean and diffusion distances

% Euclidean distances
PeakOpts.ModeDetection='Euclidean';

tic;
[CoresLearnedEuclidean, CoreLabelsTupleEuclidean, CentersEuclidean, G_euc, ExtendedCentersEuclidean, DistStructEuclidean, ~] = DensityPeaksEstimation(X, K_Learned, CoreSize, PeakOpts);
TimeFSFDPC_1=toc;

% Diffusion distances
PeakOpts.ModeDetection='Diffusion';

tic;
[CoresLearnedDiffusion, CoreLabelsTupleDiffusion, CentersDiffusion, G_dif, ExtendedCentersDiffusion, DistStructDiffusion, G_iterated] = DensityPeaksEstimation(X, K_Learned, CoreSize, PeakOpts);
TimeDL_1=toc;

%% Compute features for comparison methods

%Principal components

tic;
V=GetPC(X);
TimePCA=toc;

%Independent components
tic;
ICA=fastica(X', 'firsteig',1,'lasteig',K_Learned, 'verbose','off','stabilization','on');
TimeICA=toc;

%Gaussian random projection
tic;
RP= randn(size(X,2),K_Learned)/sqrt(K_Learned);
RP=RP./sqrt(sum(RP.^2));
TimeRP=toc;

%Eigenvectors of normalized Laplacian
tic;
[EigVecsLE, EigValsLE]=SpectralEmbeddingJMM(X,K_Learned,PeakOpts.DiffusionOpts.kNN,PeakOpts.DiffusionOpts.epsilon);
TimeSC=toc;

%SMCE
tic;
EigVecsSMCE=SparseSubspaceClustering(X,Y,K_GT);
TimeSMCE=toc;

%% PLSR with diffusion learned cores

% Compute the PLS embeddding and regression coefficient
[~,~,~,~,betaDiffusionPLS]=plsregress(X(CoresLearnedDiffusion(:),:),CoreLabelsTupleDiffusion,K_Learned,'cv',10);

%Compute class labels by running kmeans on the labels learned from PLS
%regression.
ResponsesPLSRCoresDiffusion=[ones(size(X,1),1),X]*betaDiffusionPLS;

%% PLSR with Euclidean learned cores

% Compute the PLS embeddding and regression coefficient
[XL_learned,~,~,~,betaEuclideanPLS]=plsregress(X(CoresLearnedEuclidean(:),:),CoreLabelsTupleEuclidean,K_Learned,'cv',10);

%Compute class labels by running kmeans on the labels learned from PLS
%regression.
ResponsesPLSRCoresEuclidean=[ones(size(X,1),1),X]*betaEuclideanPLS;

%% Compute all labels

ComputeLabels;

%% Compute OA, AA, Kappa after aligning

QuantitativeAnalysis;

%%  Generate all plots

if UserPlot==1
    MakePlots
end

