%% Make Plots for all data

close all;

%% HSI examples

if strcmp(DataInput.datasource,'HSI')
    
    if isempty(GT)
        GT=reshape(LabelsRaw,M,N);
    end
    
    close all;
    
    [I,J]=find(GT>0);
    
    GT=UniqueGT(GT(GT>0));
    
    GT=sparse(I,J,GT,M,N);
    
    %%  GT

    h=figure;
    imagesc(GT);
    axis equal tight;
    axis off;
  
    if SavePlots
        saveas(h,[DataInput.HSIname,'_GT.pdf']);
        system(['/usr/local/bin/pdfcrop --verbose ',DataInput.HSIname,'_GT.pdf']);
        savefig([DataInput.HSIname,'_GT']);
    end
       
   
    %% Kmeans
    
    h=figure;
    imagesc(sparse(I,J,AlignClusters(GT(GT>0)',LabelsRaw(GT>0),K_Learned),M,N));
    axis off; 
    axis equal tight;
    
    if SavePlots==1
        saveas(h,[DataInput.HSIname,'_Kmeans.pdf']);
        system(['/usr/local/bin/pdfcrop --verbose ',DataInput.HSIname,'_Kmeans.pdf']);
        savefig([DataInput.HSIname,'_Kmeans']);
    end
    
    %% PCA
   
    h=figure; 
    imagesc(sparse(I,J,AlignClusters(GT(GT>0)',LabelsPCA(GT>0),K_Learned),M,N));
    axis off; 
    axis equal tight;    
    
    if SavePlots==1
        saveas(h,[DataInput.HSIname,'_PCA.pdf']);
        system(['/usr/local/bin/pdfcrop --verbose ',DataInput.HSIname,'_PCA.pdf']);
        savefig([DataInput.HSIname,'_PCA']);
    end
    
    %% ICA
    
    h=figure; 
    imagesc(sparse(I,J,AlignClusters(GT(GT>0)',LabelsICA(GT>0),K_Learned),M,N));
    axis off; 
    axis equal tight;     
    
    if SavePlots==1
        saveas(h,[DataInput.HSIname,'_ICA.pdf']);
        system(['/usr/local/bin/pdfcrop --verbose ',DataInput.HSIname,'_ICA.pdf']); 
        savefig([DataInput.HSIname,'_ICA']);
    end
    
     %% Random Projection
    
    h=figure; 
    imagesc(sparse(I,J,AlignClusters(GT(GT>0)',LabelsRandProj(GT>0),K_Learned),M,N));
    axis off; 
    axis equal tight;     
    
    if SavePlots==1
        saveas(h,[DataInput.HSIname,'_RP.pdf']);
        system(['/usr/local/bin/pdfcrop --verbose ',DataInput.HSIname,'_RP.pdf']);
        savefig([DataInput.HSIname,'_RandomProj']);
    end
    
    %% Spectral Clustering
    
    h=figure; 
    imagesc(sparse(I,J,AlignClusters(GT(GT>0)',LabelsSpectral(GT>0),K_Learned),M,N));
    axis off; 
    axis equal tight;
    
    if SavePlots==1
        saveas(h,[DataInput.HSIname,'_SC.pdf']);
        system(['/usr/local/bin/pdfcrop --verbose ',DataInput.HSIname,'_SC.pdf']);
        savefig([DataInput.HSIname,'_SC']);
    end
    
    %% GMM
    
    h=figure;
    imagesc(sparse(I,J,AlignClusters(GT(GT>0)',LabelsGMM(GT>0),K_Learned),M,N));
    axis off;
    axis equal tight;    
    
    if SavePlots==1
        saveas(h,[DataInput.HSIname,'_GMM.pdf']);
        system(['/usr/local/bin/pdfcrop --verbose ',DataInput.HSIname,'_GMM.pdf']);
        savefig([DataInput.HSIname,'_GMM']);
    end
    
    %% SMCE
    
    h=figure;
    imagesc(sparse(I,J,AlignClusters(GT(GT>0)',LabelsSMCE(GT>0),K_Learned),M,N));
    axis off;
    axis equal tight;    
    
    if SavePlots==1
        saveas(h,[DataInput.HSIname,'_SMCE.pdf']);
        system(['/usr/local/bin/pdfcrop --verbose ',DataInput.HSIname,'_SMCE.pdf']);
        savefig([DataInput.HSIname,'_SMCE']);
    end
    
    %% NMF
    
    h=figure;
    imagesc(sparse(I,J,AlignClusters(GT(GT>0)',LabelsNMF(GT>0)',K_Learned),M,N));
    axis off;
    axis equal tight;    
    
    if SavePlots==1
        saveas(h,[DataInput.HSIname,'_NMF.pdf']);
        system(['/usr/local/bin/pdfcrop --verbose ',DataInput.HSIname,'_NMF.pdf']);
        savefig([DataInput.HSIname,'_NMF']);
    end
    
    %% Bertozzi
    
    h=figure;
    imagesc(sparse(I,J,AlignClusters(GT(GT>0)',LabelsBertozzi(GT>0)',K_Learned),M,N));
    axis off;
    axis equal tight;    
    
    if SavePlots==1
        saveas(h,[DataInput.HSIname,'_Bertozzi.pdf']);
        system(['/usr/local/bin/pdfcrop --verbose ',DataInput.HSIname,'_Bertozzi.pdf']);
        savefig([DataInput.HSIname,'_Bertozzi']);
    end
    
    %% FSFDPC
    
    
    h=figure; 
    imagesc(sparse(I,J,AlignClusters(GT(GT>0)',LabelsFSFDPC(GT>0),K_Learned),M,N));
    axis off;
    axis equal tight;    
    
    if SavePlots==1
        saveas(h,[DataInput.HSIname,'_FSFDPC.pdf']);
        system(['/usr/local/bin/pdfcrop --verbose ',DataInput.HSIname,'_FSFDPC.pdf']);
        savefig([DataInput.HSIname,'_FSFDPC']);
    end
    
    %% Diffusion Learning
    
    h=figure; 
    imagesc(sparse(I,J,AlignClusters(GT(GT>0)',Labels_DL(GT>0),K_Learned),M,N));
    axis off; 
    axis equal tight;     
    
    if SavePlots==1
        saveas(h,[DataInput.HSIname,'_DL.pdf']);
        system(['/usr/local/bin/pdfcrop --verbose ',DataInput.HSIname,'_DL.pdf'])
        savefig([DataInput.HSIname,'_DL']);
    end
    
    %% Diffusion learning spatial-spectral
    
    h=figure; 
    imagesc(sparse(I,J,AlignClusters(GT(GT>0)',Labels_DLSS(GT>0),K_Learned),M,N));
    axis off; 
    axis equal tight;     
    
    if SavePlots==1
        saveas(h,[DataInput.HSIname,'_DLSS.pdf']);
        system(['/usr/local/bin/pdfcrop --verbose ',DataInput.HSIname,'_DLSS.pdf'])
        savefig([DataInput.HSIname,'_DLSS']);
    end
    
     %% Diffusion learning spatial spectral active
    
    h=figure; 
    imagesc(sparse(I,J,AlignClusters(GT(GT>0)',Labels_DLSSActive(GT>0),K_Learned),M,N));
    axis off; 
    axis equal tight;     
    
    if SavePlots==1
        saveas(h,[DataInput.HSIname,'_DLSSActive.pdf']);
        system(['/usr/local/bin/pdfcrop --verbose ',DataInput.HSIname,'_DLSSActive.pdf']);
        savefig([DataInput.HSIname,'_DLSSActive']);
    end
    
    %% Sum of all HSI bands
    
    h=figure; 
    imagesc(sum(DataParameters.RawHSI,3));
    axis off;
    axis equal tight;
    
    if SavePlots==1
        saveas(h,[DataInput.HSIname,'_BandSum.pdf']);
        system(['/usr/local/bin/pdfcrop --verbose ',DataInput.HSIname,'_BandSum.pdf']);
        savefig([DataInput.HSIname,'_BandSum']);
    end
    
    %% PLSR Euclidean cores
    
    h=figure; 
    imagesc(sparse(I,J,AlignClusters(GT(GT>0)',LabelsPLSR_EuclideanCores(GT>0),K_GT),M,N));
    axis off; 
    axis equal tight; 
    
    if SavePlots==1
        saveas(h,[DataInput.HSIname,'_PLSREC.pdf']);
        system(['/usr/local/bin/pdfcrop --verbose ',DataInput.HSIname,'_PLSREC.pdf']);
        savefig([DataInput.HSIname,'_PLSRDC']);
    end
    
    %% PLSR diffusion cores
    
    h=figure; 
    imagesc(sparse(I,J,AlignClusters(GT(GT>0)',LabelsPLSR_DiffusionCores(GT>0),K_Learned),M,N));
    axis off; 
    axis equal tight;     
    
    if SavePlots==1
        saveas(h,[DataInput.HSIname,'_PLSRDC.pdf']);
        system(['/usr/local/bin/pdfcrop --verbose ',DataInput.HSIname,'_PLSRDC.pdf']);
        savefig([DataInput.HSIname,'_PLSRDC']);
    end
    
end

%% Plot sorted DeltaDensity from diffusion distances

h=figure;
plot(1:length(ExtendedCentersDiffusion),DistStructDiffusion.DeltaDensity(ExtendedCentersDiffusion),K_GT,DistStructDiffusion.DeltaDensity(ExtendedCentersDiffusion(K_GT)),'r*');

if SavePlots==1
    saveas(h,[DataInput.HSIname,'_Kink.pdf']);
    system(['/usr/local/bin/pdfcrop --verbose ',DataInput.HSIname,'_Kink.pdf'])
    savefig([DataInput.HSIname,'_Kink']);
end

Diff=-diff(DistStructDiffusion.DeltaDensity(ExtendedCentersDiffusion));

h=figure;
plot(1:length(ExtendedCentersDiffusion)-1,Diff,K_GT,Diff(K_GT),'r*'); %title('Gap in DeltaDensity, with ground truth class number in red'); 

if SavePlots==1
    saveas(h,[DataInput.HSIname,'_Diff.pdf']);
    system(['/usr/local/bin/pdfcrop --verbose ',DataInput.HSIname,'_Diff.pdf']);
    savefig([DataInput.HSIname,'_Diff']);
end